@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2020 by mod_tile contributors (see AUTHORS file)
+ * Copyright (c) 2007 - 2023 by mod_tile contributors (see AUTHORS file)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,6 +34,7 @@
 #include <strings.h>
 #include <getopt.h>
 
+#include "config.h"
 #include "render_config.h"
 #include "daemon.h"
 #include "gen_tile.h"
@@ -43,11 +44,13 @@
 #include "g_logger.h"
 
 // extern "C" {
+#ifdef HAVE_INIPARSER_INIPARSER_H
 #include <iniparser/iniparser.h>
+#else
+#include <iniparser.h>
+#endif
 // }
 
-#define PIDFILE "/run/renderd/renderd.pid"
-
 #define PFD_LISTEN        0
 #define PFD_EXIT_PIPE     1
 #define PFD_SPECIAL_COUNT 2
@@ -739,14 +742,16 @@ int main(int argc, char **argv)
 	while (1) {
 		int option_index = 0;
 		static struct option long_options[] = {
-			{"config", required_argument, 0, 'c'},
-			{"foreground", no_argument, 0, 'f'},
-			{"slave", required_argument, 0, 's'},
-			{"help", no_argument, 0, 'h'},
+			{"config",     required_argument, 0, 'c'},
+			{"foreground", no_argument,       0, 'f'},
+			{"slave",      required_argument, 0, 's'},
+
+			{"help",       no_argument,       0, 'h'},
+			{"version",    no_argument,       0, 'V'},
 			{0, 0, 0, 0}
 		};
 
-		c = getopt_long(argc, argv, "hfc:", long_options, &option_index);
+		c = getopt_long(argc, argv, "c:fs:hV", long_options, &option_index);
 
 		if (c == -1) {
 			break;
@@ -773,10 +778,16 @@ int main(int argc, char **argv)
 			case 'h':
 				fprintf(stderr, "Usage: renderd [OPTION] ...\n");
 				fprintf(stderr, "Mapnik rendering daemon\n");
-				fprintf(stderr, "  -f, --foreground      run in foreground\n");
-				fprintf(stderr, "  -h, --help            display this help and exit\n");
 				fprintf(stderr, "  -c, --config=CONFIG   set location of config file (default %s)\n", RENDERD_CONFIG);
+				fprintf(stderr, "  -f, --foreground      run in foreground\n");
 				fprintf(stderr, "  -s, --slave=CONFIG_NR set which render slave this is (default 0)\n");
+				fprintf(stderr, "\n");
+				fprintf(stderr, "  -h, --help            display this help and exit\n");
+				fprintf(stderr, "  -V, --version         display the version number and exit\n");
+				exit(0);
+
+			case 'V':
+				fprintf(stdout, "%s\n", VERSION);
 				exit(0);
 
 			default:
@@ -790,8 +801,9 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	g_logger(G_LOG_LEVEL_INFO, "Rendering daemon started");
+	g_logger(G_LOG_LEVEL_INFO, "Rendering daemon started (version %s)", VERSION);
 
+	g_logger(G_LOG_LEVEL_INFO, "Initialising request queue");
 	render_request_queue = request_queue_init();
 
 	if (render_request_queue == NULL) {
@@ -799,8 +811,6 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	g_logger(G_LOG_LEVEL_INFO, "Initialising request_queue");
-
 	xmlconfigitem maps[XMLCONFIGS_MAX];
 	bzero(maps, sizeof(xmlconfigitem) * XMLCONFIGS_MAX);
 
@@ -808,9 +818,11 @@ int main(int argc, char **argv)
 	bzero(config_slaves, sizeof(renderd_config) * MAX_SLAVES);
 	bzero(&config, sizeof(renderd_config));
 
+	g_logger(G_LOG_LEVEL_INFO, "Parsing config file: %s", config_file_name);
 	dictionary *ini = iniparser_load(config_file_name);
 
-	if (! ini) {
+	if (!ini) {
+		g_logger(G_LOG_LEVEL_CRITICAL, "Failed to load config file: %s", config_file_name);
 		exit(1);
 	}
 
@@ -819,19 +831,86 @@ int main(int argc, char **argv)
 	int iconf = -1;
 	char buffer[PATH_MAX];
 
+	g_logger(G_LOG_LEVEL_DEBUG, "Parsing renderd config section(s)");
+
 	for (int section = 0; section < iniparser_getnsec(ini); section++) {
-		char *name = iniparser_getsecname(ini, section);
-		g_logger(G_LOG_LEVEL_INFO, "Parsing section %s", name);
+		const char *name = iniparser_getsecname(ini, section);
+
+		if (strncmp(name, "renderd", 7) == 0) {
+			/* this is a renderd config section */
+			int render_sec = 0;
+
+			if (sscanf(name, "renderd%i", &render_sec) != 1) {
+				render_sec = 0;
+			}
+
+			g_logger(G_LOG_LEVEL_DEBUG, "Parsing renderd config section %i: %s", render_sec, name);
+
+			if (render_sec >= MAX_SLAVES) {
+				g_logger(G_LOG_LEVEL_CRITICAL, "Can't handle more than %i renderd config sections",
+					 MAX_SLAVES);
+				exit(7);
+			}
+
+			snprintf(buffer, sizeof(buffer), "%s:socketname", name);
+			config_slaves[render_sec].socketname = iniparser_getstring(ini,
+							       buffer, (char *) RENDERD_SOCKET);
+			snprintf(buffer, sizeof(buffer), "%s:iphostname", name);
+			config_slaves[render_sec].iphostname = iniparser_getstring(ini,
+							       buffer, "");
+			snprintf(buffer, sizeof(buffer), "%s:ipport", name);
+			config_slaves[render_sec].ipport = iniparser_getint(ini, buffer, 0);
+			snprintf(buffer, sizeof(buffer), "%s:num_threads", name);
+			config_slaves[render_sec].num_threads = iniparser_getint(ini,
+								buffer, NUM_THREADS);
+			snprintf(buffer, sizeof(buffer), "%s:tile_dir", name);
+			config_slaves[render_sec].tile_dir = iniparser_getstring(ini,
+							     buffer, (char *) RENDERD_TILE_DIR);
+			snprintf(buffer, sizeof(buffer), "%s:stats_file", name);
+			config_slaves[render_sec].stats_filename = iniparser_getstring(ini,
+					buffer, NULL);
+			snprintf(buffer, sizeof(buffer), "%s:pid_file", name);
+			config_slaves[render_sec].pid_filename = iniparser_getstring(ini,
+					buffer, (char *) RENDERD_PIDFILE);
+
+			if (render_sec == active_slave) {
+				config.socketname = config_slaves[render_sec].socketname;
+				config.iphostname = config_slaves[render_sec].iphostname;
+				config.ipport = config_slaves[render_sec].ipport;
+				config.num_threads = config_slaves[render_sec].num_threads;
+				config.tile_dir = config_slaves[render_sec].tile_dir;
+				config.stats_filename
+					= config_slaves[render_sec].stats_filename;
+				config.pid_filename
+					= config_slaves[render_sec].pid_filename;
+				config.mapnik_plugins_dir = iniparser_getstring(ini,
+							    "mapnik:plugins_dir", (char *) MAPNIK_PLUGINS_DIR);
+				config.mapnik_font_dir = iniparser_getstring(ini,
+							 "mapnik:font_dir", (char *) MAPNIK_FONTS_DIR);
+				config.mapnik_font_dir_recurse = iniparser_getboolean(ini,
+								 "mapnik:font_dir_recurse", MAPNIK_FONTS_DIR_RECURSE);
+			} else {
+				noSlaveRenders += config_slaves[render_sec].num_threads;
+			}
+		}
+	}
+
+	g_logger(G_LOG_LEVEL_DEBUG, "Parsing map config section(s)");
+
+	for (int section = 0; section < iniparser_getnsec(ini); section++) {
+		const char *name = iniparser_getsecname(ini, section);
 
 		if (strncmp(name, "renderd", 7) && strcmp(name, "mapnik")) {
-			if (config.tile_dir == NULL) {
+			/* this is a map config section */
+			if (config.num_threads == 0 || config.tile_dir == NULL) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "No valid (active) renderd config section available");
 				exit(7);
 			}
 
-			/* this is a map section */
 			iconf++;
 
+			g_logger(G_LOG_LEVEL_DEBUG, "Parsing map config section %i: %s", iconf, name);
+
 			if (iconf >= XMLCONFIGS_MAX) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "Config: more than %d configurations found", XMLCONFIGS_MAX);
 				exit(7);
@@ -844,8 +923,8 @@ int main(int argc, char **argv)
 
 			strcpy(maps[iconf].xmlname, name);
 
-			sprintf(buffer, "%s:uri", name);
-			char *ini_uri = iniparser_getstring(ini, buffer, (char *)"");
+			snprintf(buffer, sizeof(buffer), "%s:uri", name);
+			const char *ini_uri = iniparser_getstring(ini, buffer, (char *)"");
 
 			if (strlen(ini_uri) >= (PATH_MAX - 1)) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "URI too long: %s", ini_uri);
@@ -854,8 +933,8 @@ int main(int argc, char **argv)
 
 			strcpy(maps[iconf].xmluri, ini_uri);
 
-			sprintf(buffer, "%s:xml", name);
-			char *ini_xmlpath = iniparser_getstring(ini, buffer, (char *)"");
+			snprintf(buffer, sizeof(buffer), "%s:xml", name);
+			const char *ini_xmlpath = iniparser_getstring(ini, buffer, (char *)"");
 
 			if (strlen(ini_xmlpath) >= (PATH_MAX - 1)) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "XML path too long: %s", ini_xmlpath);
@@ -864,8 +943,8 @@ int main(int argc, char **argv)
 
 			strcpy(maps[iconf].xmlfile, ini_xmlpath);
 
-			sprintf(buffer, "%s:host", name);
-			char *ini_hostname = iniparser_getstring(ini, buffer, (char *) "");
+			snprintf(buffer, sizeof(buffer), "%s:host", name);
+			const char *ini_hostname = iniparser_getstring(ini, buffer, (char *) "");
 
 			if (strlen(ini_hostname) >= (PATH_MAX - 1)) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "Host name too long: %s", ini_hostname);
@@ -874,8 +953,8 @@ int main(int argc, char **argv)
 
 			strcpy(maps[iconf].host, ini_hostname);
 
-			sprintf(buffer, "%s:htcphost", name);
-			char *ini_htcpip = iniparser_getstring(ini, buffer, (char *) "");
+			snprintf(buffer, sizeof(buffer), "%s:htcphost", name);
+			const char *ini_htcpip = iniparser_getstring(ini, buffer, (char *) "");
 
 			if (strlen(ini_htcpip) >= (PATH_MAX - 1)) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "HTCP host name too long: %s", ini_htcpip);
@@ -884,8 +963,8 @@ int main(int argc, char **argv)
 
 			strcpy(maps[iconf].htcpip, ini_htcpip);
 
-			sprintf(buffer, "%s:tilesize", name);
-			char *ini_tilesize = iniparser_getstring(ini, buffer, (char *) "256");
+			snprintf(buffer, sizeof(buffer), "%s:tilesize", name);
+			const char *ini_tilesize = iniparser_getstring(ini, buffer, (char *) "256");
 			maps[iconf].tile_px_size = atoi(ini_tilesize);
 
 			if (maps[iconf].tile_px_size < 1) {
@@ -893,8 +972,8 @@ int main(int argc, char **argv)
 				exit(7);
 			}
 
-			sprintf(buffer, "%s:scale", name);
-			char *ini_scale = iniparser_getstring(ini, buffer, (char *) "1.0");
+			snprintf(buffer, sizeof(buffer), "%s:scale", name);
+			const char *ini_scale = iniparser_getstring(ini, buffer, (char *) "1.0");
 			maps[iconf].scale_factor = atof(ini_scale);
 
 			if (maps[iconf].scale_factor < 0.1 || maps[iconf].scale_factor > 8.0) {
@@ -902,8 +981,8 @@ int main(int argc, char **argv)
 				exit(7);
 			}
 
-			sprintf(buffer, "%s:tiledir", name);
-			char *ini_tiledir = iniparser_getstring(ini, buffer, (char *) config.tile_dir);
+			snprintf(buffer, sizeof(buffer), "%s:tiledir", name);
+			const char *ini_tiledir = iniparser_getstring(ini, buffer, (char *) config.tile_dir);
 
 			if (strlen(ini_tiledir) >= (PATH_MAX - 1)) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "Tiledir too long: %s", ini_tiledir);
@@ -912,21 +991,21 @@ int main(int argc, char **argv)
 
 			strcpy(maps[iconf].tile_dir, ini_tiledir);
 
-			sprintf(buffer, "%s:maxzoom", name);
-			char *ini_maxzoom = iniparser_getstring(ini, buffer, "18");
+			snprintf(buffer, sizeof(buffer), "%s:maxzoom", name);
+			const char *ini_maxzoom = iniparser_getstring(ini, buffer, "18");
 			maps[iconf].max_zoom = atoi(ini_maxzoom);
 
 			if (maps[iconf].max_zoom > MAX_ZOOM) {
-				g_logger(G_LOG_LEVEL_CRITICAL, "Specified max zoom (%i) is to large. Renderd currently only supports up to zoom level %i", maps[iconf].max_zoom, MAX_ZOOM);
+				g_logger(G_LOG_LEVEL_CRITICAL, "Specified max zoom (%i) is too large. Renderd currently only supports up to zoom level %i", maps[iconf].max_zoom, MAX_ZOOM);
 				exit(7);
 			}
 
-			sprintf(buffer, "%s:minzoom", name);
-			char *ini_minzoom = iniparser_getstring(ini, buffer, "0");
+			snprintf(buffer, sizeof(buffer), "%s:minzoom", name);
+			const char *ini_minzoom = iniparser_getstring(ini, buffer, "0");
 			maps[iconf].min_zoom = atoi(ini_minzoom);
 
 			if (maps[iconf].min_zoom < 0) {
-				g_logger(G_LOG_LEVEL_CRITICAL, "Specified min zoom (%i) is to small. Minimum zoom level has to be greater or equal to 0", maps[iconf].min_zoom);
+				g_logger(G_LOG_LEVEL_CRITICAL, "Specified min zoom (%i) is too small. Minimum zoom level has to be greater or equal to 0", maps[iconf].min_zoom);
 				exit(7);
 			}
 
@@ -935,8 +1014,8 @@ int main(int argc, char **argv)
 				exit(7);
 			}
 
-			sprintf(buffer, "%s:parameterize_style", name);
-			char *ini_parameterize = iniparser_getstring(ini, buffer, "");
+			snprintf(buffer, sizeof(buffer), "%s:parameterize_style", name);
+			const char *ini_parameterize = iniparser_getstring(ini, buffer, "");
 
 			if (strlen(ini_parameterize) >= (PATH_MAX - 1)) {
 				g_logger(G_LOG_LEVEL_CRITICAL, "Parameterize_style too long: %s", ini_parameterize);
@@ -945,61 +1024,21 @@ int main(int argc, char **argv)
 
 			strcpy(maps[iconf].parameterization, ini_parameterize);
 
+			snprintf(buffer, sizeof(buffer), "%s:type", name);
+			const char *ini_type = iniparser_getstring(ini, buffer, "png image/png png256");
+
+			const char ini_fileExtension[INILINE_MAX] = "png";
+			const char ini_mimeType[INILINE_MAX] = "image/png";
+			const char ini_outputFormat[INILINE_MAX] = "png256";
+
+			sscanf(ini_type, "%[^ ] %[^ ] %[^;#]", ini_fileExtension, ini_mimeType, ini_outputFormat);
+
+			strcpy(maps[iconf].output_format, ini_outputFormat);
+
 			/* Pass this information into the rendering threads,
 			 * as it is needed to configure mapniks number of connections
 			 */
 			maps[iconf].num_threads = config.num_threads;
-
-		} else if (strncmp(name, "renderd", 7) == 0) {
-			int render_sec = 0;
-
-			if (sscanf(name, "renderd%i", &render_sec) != 1) {
-				render_sec = 0;
-			}
-
-			g_logger(G_LOG_LEVEL_INFO, "Parsing render section %i", render_sec);
-
-			if (render_sec >= MAX_SLAVES) {
-				g_logger(G_LOG_LEVEL_CRITICAL, "Can't handle more than %i render sections",
-					 MAX_SLAVES);
-				exit(7);
-			}
-
-			sprintf(buffer, "%s:socketname", name);
-			config_slaves[render_sec].socketname = iniparser_getstring(ini,
-							       buffer, (char *) RENDER_SOCKET);
-			sprintf(buffer, "%s:iphostname", name);
-			config_slaves[render_sec].iphostname = iniparser_getstring(ini,
-							       buffer, "");
-			sprintf(buffer, "%s:ipport", name);
-			config_slaves[render_sec].ipport = iniparser_getint(ini, buffer, 0);
-			sprintf(buffer, "%s:num_threads", name);
-			config_slaves[render_sec].num_threads = iniparser_getint(ini,
-								buffer, NUM_THREADS);
-			sprintf(buffer, "%s:tile_dir", name);
-			config_slaves[render_sec].tile_dir = iniparser_getstring(ini,
-							     buffer, (char *) HASH_PATH);
-			sprintf(buffer, "%s:stats_file", name);
-			config_slaves[render_sec].stats_filename = iniparser_getstring(ini,
-					buffer, NULL);
-
-			if (render_sec == active_slave) {
-				config.socketname = config_slaves[render_sec].socketname;
-				config.iphostname = config_slaves[render_sec].iphostname;
-				config.ipport = config_slaves[render_sec].ipport;
-				config.num_threads = config_slaves[render_sec].num_threads;
-				config.tile_dir = config_slaves[render_sec].tile_dir;
-				config.stats_filename
-					= config_slaves[render_sec].stats_filename;
-				config.mapnik_plugins_dir = iniparser_getstring(ini,
-							    "mapnik:plugins_dir", (char *) MAPNIK_PLUGINS);
-				config.mapnik_font_dir = iniparser_getstring(ini,
-							 "mapnik:font_dir", (char *) FONT_DIR);
-				config.mapnik_font_dir_recurse = iniparser_getboolean(ini,
-								 "mapnik:font_dir_recurse", FONT_RECURSE);
-			} else {
-				noSlaveRenders += config_slaves[render_sec].num_threads;
-			}
 		}
 	}
 
@@ -1017,6 +1056,7 @@ int main(int argc, char **argv)
 
 	g_logger(G_LOG_LEVEL_INFO, "config renderd: tile_dir=%s", config.tile_dir);
 	g_logger(G_LOG_LEVEL_INFO, "config renderd: stats_file=%s", config.stats_filename);
+	g_logger(G_LOG_LEVEL_INFO, "config renderd: pid_file=%s", config.pid_filename);
 	g_logger(G_LOG_LEVEL_INFO, "config mapnik:  plugins_dir=%s", config.mapnik_plugins_dir);
 	g_logger(G_LOG_LEVEL_INFO, "config mapnik:  font_dir=%s", config.mapnik_font_dir);
 	g_logger(G_LOG_LEVEL_INFO, "config mapnik:  font_dir_recurse=%d", config.mapnik_font_dir_recurse);
@@ -1044,13 +1084,15 @@ int main(int argc, char **argv)
 			 config_slaves[i].tile_dir);
 		g_logger(G_LOG_LEVEL_INFO, "config renderd(%i): stats_file=%s", i,
 			 config_slaves[i].stats_filename);
+		g_logger(G_LOG_LEVEL_INFO, "config renderd(%i): pid_file=%s", i,
+			 config_slaves[i].pid_filename);
 	}
 
 	for (iconf = 0; iconf < XMLCONFIGS_MAX; ++iconf) {
 		if (maps[iconf].xmlname[0] != 0) {
-			g_logger(G_LOG_LEVEL_INFO, "config map %d:   name(%s) file(%s) uri(%s) htcp(%s) host(%s)",
+			g_logger(G_LOG_LEVEL_INFO, "config map %d:   name(%s) file(%s) uri(%s) output_format(%s) htcp(%s) host(%s)",
 				 iconf, maps[iconf].xmlname, maps[iconf].xmlfile, maps[iconf].xmluri,
-				 maps[iconf].htcpip, maps[iconf].host);
+				 maps[iconf].output_format, maps[iconf].htcpip, maps[iconf].host);
 		}
 	}
 
@@ -1085,7 +1127,7 @@ int main(int argc, char **argv)
 		}
 
 		/* write pid file */
-		FILE *pidfile = fopen(PIDFILE, "w");
+		FILE *pidfile = fopen(config.pid_filename, "w");
 
 		if (pidfile) {
 			(void) fprintf(pidfile, "%d\n", getpid());
