@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2020 by mod_tile contributors (see AUTHORS file)
+ * Copyright (c) 2007 - 2023 by mod_tile contributors (see AUTHORS file)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -67,9 +67,13 @@
 #define box2d Envelope
 #define zoom_to_box zoomToBox
 #else
+#if MAPNIK_VERSION >= 400000
+#include <mapnik/geometry/box2d.hpp>
+#else
 #include <mapnik/box2d.hpp>
 #endif
 #endif
+#endif
 
 
 using namespace mapnik;
@@ -100,6 +104,7 @@ struct projectionconfig {
 struct xmlmapconfig {
 	char xmlname[XMLCONFIG_MAX];
 	char xmlfile[PATH_MAX];
+	char output_format[XMLCONFIG_MAX];
 	struct storage_backend * store;
 	Map map;
 	struct projectionconfig * prj;
@@ -295,14 +300,19 @@ static enum protoCmd render(struct xmlmapconfig * map, int x, int y, int z, char
 	mapnik::image_32 buf(render_size_tx * map->tilesize, render_size_ty * map->tilesize);
 
 	try {
-		Map map_parameterized = map->map;
-
 		if (map->parameterize_function) {
-			map->parameterize_function(map_parameterized, options);
-		}
+			Map map_parameterized = map->map;
 
-		mapnik::agg_renderer<mapnik::image_32> ren(map_parameterized, buf, map->scale);
-		ren.apply();
+			map->parameterize_function(map_parameterized, options);
+
+			map_parameterized.load_fonts();
+
+			mapnik::agg_renderer<mapnik::image_32> ren(map_parameterized, buf, map->scale);
+			ren.apply();
+		} else {
+			mapnik::agg_renderer<mapnik::image_32> ren(map->map, buf, map->scale);
+			ren.apply();
+		}
 	} catch (std::exception const& ex) {
 		g_logger(G_LOG_LEVEL_ERROR, "failed to render TILE %s %d %d-%d %d-%d", map->xmlname, z, x, x + render_size_tx - 1, y, y + render_size_ty - 1);
 		g_logger(G_LOG_LEVEL_ERROR, "  reason: %s", ex.what());
@@ -320,14 +330,14 @@ static enum protoCmd render(struct xmlmapconfig * map, int x, int y, int z, char
 #else
 			mapnik::image_view<mapnik::image_data_32> vw(xx * map->tilesize, yy * map->tilesize, map->tilesize, map->tilesize, buf.data());
 #endif
-			tiles.set(xx, yy, save_to_string(vw, "png256"));
+			tiles.set(xx, yy, save_to_string(vw, map->output_format));
 		}
 	}
 
 	return cmdDone; // OK
 }
 #else //METATILE
-static enum protoCmd render(Map &m, const char *tile_dir, char *xmlname, projection &prj, int x, int y, int z)
+static enum protoCmd render(Map &m, const char *tile_dir, char *xmlname, projection &prj, int x, int y, int z, char* outputFormat)
 {
 	char filename[PATH_MAX];
 	char tmp[PATH_MAX];
@@ -361,7 +371,7 @@ static enum protoCmd render(Map &m, const char *tile_dir, char *xmlname, project
 
 	mapnik::image_view<mapnik::image_data_32> vw(128, 128, 256, 256, buf.data());
 	g_logger(G_LOG_LEVEL_DEBUG, "Render %i %i %i %s", z, x, y, filename)
-	mapnik::save_to_file(vw, tmp, "png256");
+	mapnik::save_to_file(vw, tmp, outputFormat);
 
 	if (rename(tmp, filename)) {
 		perror(tmp);
@@ -399,6 +409,7 @@ void *render_thread(void * arg)
 
 		strcpy(maps[iMaxConfigs].xmlname, parentxmlconfig[iMaxConfigs].xmlname);
 		strcpy(maps[iMaxConfigs].xmlfile, parentxmlconfig[iMaxConfigs].xmlfile);
+		strcpy(maps[iMaxConfigs].output_format, parentxmlconfig[iMaxConfigs].output_format);
 		maps[iMaxConfigs].store = init_storage_backend(parentxmlconfig[iMaxConfigs].tile_dir);
 		maps[iMaxConfigs].tilesize  = parentxmlconfig[iMaxConfigs].tile_px_size;
 		maps[iMaxConfigs].scale  = parentxmlconfig[iMaxConfigs].scale_factor;
@@ -415,6 +426,10 @@ void *render_thread(void * arg)
 			try {
 				mapnik::load_map(maps[iMaxConfigs].map, maps[iMaxConfigs].xmlfile);
 
+				if (!maps[iMaxConfigs].parameterize_function) {
+					maps[iMaxConfigs].map.load_fonts();
+				}
+
 				/* If we have more than 10 rendering threads configured, we need to fix
 				 * up the mapnik datasources to support larger postgres connection pools
 				 */
@@ -517,7 +532,7 @@ void *render_thread(void * arg)
 							}
 
 #else //METATILE
-			ret = render(maps[i].map, maps[i].tile_dir, req->xmlname, maps[i].prj, req->x, req->y, req->z);
+			ret = render(maps[i].map, maps[i].tile_dir, req->xmlname, maps[i].prj, req->x, req->y, req->z, maps[i].output_format);
 #ifdef HTCP_EXPIRE_CACHE
 			cache_expire(maps[i].htcpsock, maps[i].host, maps[i].xmluri, req->x, req->y, req->z);
 #endif
