@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2020 by mod_tile contributors (see AUTHORS file)
+ * Copyright (c) 2007 - 2023 by mod_tile contributors (see AUTHORS file)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,11 +17,12 @@
 
 #include <iostream>
 
-// https://github.com/philsquared/Catch/wiki/Supplying-your-own-main()
+// https://github.com/catchorg/Catch2/blob/v2.13.9/docs/own-main.md#let-catch2-take-full-control-of-args-and-config
 #define CATCH_CONFIG_RUNNER
-#include "catch.hpp"
+#include "catch/catch.hpp"
 
 #include "metatile.h"
+#include "config.h"
 #include "gen_tile.h"
 #include "render_config.h"
 #include "request_queue.h"
@@ -40,6 +41,10 @@
 #include <mach/clock.h>
 #include <mach/mach.h>
 #endif
+#ifdef __FreeBSD__
+#include <pthread.h>
+#include <sys/wait.h>
+#endif
 
 #include <mapnik/version.hpp>
 #if MAPNIK_VERSION < 200000
@@ -49,8 +54,12 @@
 #define box2d Envelope
 #define zoom_to_box zoomToBox
 #else
+#if MAPNIK_VERSION >= 400000
+#include <mapnik/geometry/box2d.hpp>
+#else
 #include <mapnik/box2d.hpp>
 #endif
+#endif
 
 
 #define NO_QUEUE_REQUESTS 9
@@ -103,6 +112,8 @@ void * addition_thread(void * arg)
 	uint64_t threadid;
 #ifdef __MACH__ // Mac OS X does not support SYS_gettid
 	pthread_threadid_np(NULL, &threadid);
+#elif __FreeBSD__ // FreeBSD does not support SYS_getid either
+	threadid = (uint64_t) pthread_self();
 #else
 	threadid = syscall(SYS_gettid);
 #endif
@@ -131,6 +142,150 @@ void * fetch_thread(void * arg)
 	return NULL;
 }
 
+TEST_CASE("render_expired", "render expired")
+{
+
+	SECTION("render_expired startup --help", "should start and show help message") {
+		// flawfinder: ignore
+		int ret = system("./render_expired -h");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 0);
+	}
+
+	SECTION("render_expired startup --version", "should start and show version number") {
+		// flawfinder: ignore
+		FILE *pipe = popen("./render_expired -V", "r");
+		std::string output;
+		char buffer[sizeof(VERSION)];
+		fgets(buffer, sizeof(buffer), pipe);
+		output += buffer;
+		pclose(pipe);
+		REQUIRE(output == VERSION);
+	}
+
+	SECTION("render_expired startup unrecognized option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_expired --doesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+
+	SECTION("render_expired startup invalid option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_expired -oesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+}
+
+TEST_CASE("render_list", "render list")
+{
+
+	SECTION("render_list startup --help", "should start and show help message") {
+		// flawfinder: ignore
+		int ret = system("./render_list -h");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 0);
+	}
+
+	SECTION("render_list startup --version", "should start and show version number") {
+		// flawfinder: ignore
+		FILE *pipe = popen("./render_list -V", "r");
+		std::string output;
+		char buffer[sizeof(VERSION)];
+		fgets(buffer, sizeof(buffer), pipe);
+		output += buffer;
+		pclose(pipe);
+		REQUIRE(output == VERSION);
+	}
+
+	SECTION("render_list startup unrecognized option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_list --doesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+
+	SECTION("render_list startup invalid option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_list -doesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+}
+
+TEST_CASE("render_old", "render old")
+{
+
+	SECTION("render_old startup --help", "should start and show help message") {
+		// flawfinder: ignore
+		int ret = system("./render_old -h");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 0);
+	}
+
+	SECTION("render_old startup --version", "should start and show version number") {
+		// flawfinder: ignore
+		FILE *pipe = popen("./render_old -V", "r");
+		std::string output;
+		char buffer[sizeof(VERSION)];
+		fgets(buffer, sizeof(buffer), pipe);
+		output += buffer;
+		pclose(pipe);
+		REQUIRE(output == VERSION);
+	}
+
+	SECTION("render_old startup unrecognized option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_old --doesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+
+	SECTION("render_old startup invalid option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_old -doesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+}
+
+TEST_CASE("render_speedtest", "render speed test")
+{
+
+	SECTION("render_speedtest startup --help", "should start and show help message") {
+		// flawfinder: ignore
+		int ret = system("./render_speedtest -h");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 0);
+	}
+
+	SECTION("render_speedtest startup --version", "should start and show version number") {
+		// flawfinder: ignore
+		FILE *pipe = popen("./render_speedtest -V", "r");
+		std::string output;
+		char buffer[sizeof(VERSION)];
+		fgets(buffer, sizeof(buffer), pipe);
+		output += buffer;
+		pclose(pipe);
+		REQUIRE(output == VERSION);
+	}
+
+	SECTION("render_speedtest startup unrecognized option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_speedtest --doesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+
+	SECTION("render_speedtest startup invalid option", "should return 1") {
+		// flawfinder: ignore
+		int ret = system("./render_speedtest -doesnotexit");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+}
+
 TEST_CASE("renderd/queueing", "request queueing")
 {
 	SECTION("renderd/queueing/initialisation", "test the initialisation of the request queue") {
@@ -589,23 +744,41 @@ TEST_CASE("renderd", "tile generation")
 	}
 
 	SECTION("renderd startup --help", "should start and show help message") {
+		// flawfinder: ignore
 		int ret = system("./renderd -h");
 		ret = WEXITSTATUS(ret);
-		//CAPTURE( ret );
 		REQUIRE(ret == 0);
 	}
 
+	SECTION("renderd startup --version", "should start and show version number") {
+		// flawfinder: ignore
+		FILE *pipe = popen("./renderd -V", "r");
+		std::string output;
+		char buffer[sizeof(VERSION)];
+		fgets(buffer, sizeof(buffer), pipe);
+		output += buffer;
+		pclose(pipe);
+		REQUIRE(output == VERSION);
+	}
+
+	SECTION("renderd startup --config fakefile.conf --foreground", "should not start and return 1") {
+		// flawfinder: ignore
+		int ret = system("./renderd -c fakefile.conf -f");
+		ret = WEXITSTATUS(ret);
+		REQUIRE(ret == 1);
+	}
+
 	SECTION("renderd startup unrecognized option", "should return 1") {
+		// flawfinder: ignore
 		int ret = system("./renderd --doesnotexit");
 		ret = WEXITSTATUS(ret);
-		//CAPTURE( ret );
 		REQUIRE(ret == 1);
 	}
 
 	SECTION("renderd startup invalid option", "should return 1") {
+		// flawfinder: ignore
 		int ret = system("./renderd -doesnotexit");
 		ret = WEXITSTATUS(ret);
-		//CAPTURE( ret );
 		REQUIRE(ret == 1);
 	}
 }
@@ -745,7 +918,7 @@ TEST_CASE("storage-backend", "Tile storage backend")
 
 		for (int yy = 0; yy < METATILE; yy++) {
 			for (int xx = 0; xx < METATILE; xx++) {
-				sprintf(buf, "DEADBEAF %i %i", xx, yy);
+				snprintf(buf, 8196, "DEADBEAF %i %i", xx, yy);
 				std::string tile_data(buf);
 				tiles.set(xx, yy, tile_data);
 			}
@@ -758,7 +931,7 @@ TEST_CASE("storage-backend", "Tile storage backend")
 			for (int xx = 0; xx < METATILE; xx++) {
 				tile_size = store->tile_read(store, "default", "", 1024 + METATILE + xx, 1024 + yy, 10, buf, 8195, &compressed, msg);
 				REQUIRE(tile_size == 12);
-				sprintf(buf_tmp, "DEADBEAF %i %i", xx, yy);
+				snprintf(buf_tmp, 8196, "DEADBEAF %i %i", xx, yy);
 				REQUIRE(memcmp(buf_tmp, buf, 11) == 0);
 			}
 		}
@@ -791,7 +964,7 @@ TEST_CASE("storage-backend", "Tile storage backend")
 
 		for (int yy = 0; yy < METATILE; yy++) {
 			for (int xx = 0; xx < (METATILE >> 1); xx++) {
-				sprintf(buf, "DEADBEAF %i %i", xx, yy);
+				snprintf(buf, 8196, "DEADBEAF %i %i", xx, yy);
 				std::string tile_data(buf);
 				tiles.set(xx, yy, tile_data);
 			}
@@ -808,7 +981,7 @@ TEST_CASE("storage-backend", "Tile storage backend")
 					REQUIRE(tile_size == 0);
 				} else {
 					REQUIRE(tile_size == 12);
-					sprintf(buf_tmp, "DEADBEAF %i %i", xx, yy);
+					snprintf(buf_tmp, 8196, "DEADBEAF %i %i", xx, yy);
 					REQUIRE(memcmp(buf_tmp, buf, 11) == 0);
 				}
 			}
@@ -823,10 +996,8 @@ TEST_CASE("storage-backend", "Tile storage backend")
 		struct storage_backend * store = NULL;
 		struct stat_info sinfo;
 		char * buf;
-		char * buf_tmp;
 
 		buf = (char *)malloc(8196);
-		buf_tmp = (char *)malloc(8196);
 
 		store = init_storage_backend(tile_dir);
 		REQUIRE(store != NULL);
@@ -835,7 +1006,7 @@ TEST_CASE("storage-backend", "Tile storage backend")
 
 		for (int yy = 0; yy < METATILE; yy++) {
 			for (int xx = 0; xx < METATILE; xx++) {
-				sprintf(buf, "DEADBEAF %i %i", xx, yy);
+				snprintf(buf, 8196, "DEADBEAF %i %i", xx, yy);
 				std::string tile_data(buf);
 				tiles.set(xx, yy, tile_data);
 			}
@@ -854,7 +1025,6 @@ TEST_CASE("storage-backend", "Tile storage backend")
 		REQUIRE(sinfo.size < 0);
 
 		free(buf);
-		free(buf_tmp);
 		store->close_storage(store);
 	}
 
@@ -862,10 +1032,8 @@ TEST_CASE("storage-backend", "Tile storage backend")
 		struct storage_backend * store = NULL;
 		struct stat_info sinfo;
 		char * buf;
-		char * buf_tmp;
 
 		buf = (char *)malloc(8196);
-		buf_tmp = (char *)malloc(8196);
 
 		store = init_storage_backend(tile_dir);
 		REQUIRE(store != NULL);
@@ -874,7 +1042,7 @@ TEST_CASE("storage-backend", "Tile storage backend")
 
 		for (int yy = 0; yy < METATILE; yy++) {
 			for (int xx = 0; xx < METATILE; xx++) {
-				sprintf(buf, "DEADBEAF %i %i", xx, yy);
+				snprintf(buf, 8196, "DEADBEAF %i %i", xx, yy);
 				std::string tile_data(buf);
 				tiles.set(xx, yy, tile_data);
 			}
@@ -894,7 +1062,6 @@ TEST_CASE("storage-backend", "Tile storage backend")
 		REQUIRE(sinfo.expired > 0);
 
 		free(buf);
-		free(buf_tmp);
 		store->close_storage(store);
 	}
 
@@ -985,7 +1152,8 @@ int main(int argc, char* const argv[])
 	//setvbuf(stream, 0, _IOLBF, 0); // No Buffering
 	openlog("renderd", LOG_PID | LOG_PERROR, LOG_DAEMON);
 	pthread_mutex_init(&item_counter_lock, NULL);
-	int result = Catch::Main(argc, argv);
+	int result = Catch::Session().run(argc, argv);
 	pthread_mutex_destroy(&item_counter_lock);
+	fclose(stream);
 	return result;
 }
