@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2020 by mod_tile contributors (see AUTHORS file)
+ * Copyright (c) 2007 - 2023 by mod_tile contributors (see AUTHORS file)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,32 +15,32 @@
  * along with this program; If not, see http://www.gnu.org/licenses/.
  */
 
-#include "apr.h"
-#include "apr_strings.h"
-#include "apr_thread_proc.h"    /* for RLIMIT stuff */
-#include "apr_optional.h"
-#include "apr_buckets.h"
-#include "apr_lib.h"
-#include "apr_poll.h"
+#include <apr.h>
+#include <apr_strings.h>
+#include <apr_thread_proc.h>    /* for RLIMIT stuff */
+#include <apr_optional.h>
+#include <apr_buckets.h>
+#include <apr_lib.h>
+#include <apr_poll.h>
 
 #define APR_WANT_STRFUNC
 #define APR_WANT_MEMFUNC
-#include "apr_want.h"
+#include <apr_want.h>
 
-#include "util_filter.h"
-#include "ap_config.h"
-#include "httpd.h"
-#include "http_config.h"
-#include "http_request.h"
-#include "http_core.h"
-#include "http_protocol.h"
-#include "http_main.h"
-#include "http_log.h"
-#include "util_script.h"
-#include "ap_mpm.h"
-#include "mod_core.h"
-#include "mod_cgi.h"
-#include "util_md5.h"
+#include <util_filter.h>
+#include <ap_config.h>
+#include <httpd.h>
+#include <http_config.h>
+#include <http_request.h>
+#include <http_core.h>
+#include <http_protocol.h>
+#include <http_main.h>
+#include <http_log.h>
+#include <util_script.h>
+#include <ap_mpm.h>
+#include <mod_core.h>
+#include <mod_cgi.h>
+#include <util_md5.h>
 
 module AP_MODULE_DECLARE_DATA tile_module;
 
@@ -83,6 +83,10 @@ APLOG_USE_MODULE(tile);
 #define APACHE24 1
 #endif
 
+#if (defined(__FreeBSD__) || defined(__MACH__)) && !defined(s6_addr32)
+#define s6_addr32 __u6_addr.__u6_addr32
+#endif
+
 apr_shm_t *stats_shm;
 apr_shm_t *delaypool_shm;
 char *shmfilename;
@@ -321,7 +325,7 @@ static int request_tile(request_rec *r, struct protocol *cmd, int renderImmediat
 
 				if (ret != sizeof(struct protocol_v2)) {
 					ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, r, "request_tile: Failed to read response from rendering socket. Got %d bytes but expected %d. Errno %d (%s)",
-						      ret, sizeof(struct protocol_v2), errno, strerror(errno));
+						      ret, (int) sizeof(struct protocol_v2), errno, strerror(errno));
 					break;
 				}
 
@@ -985,6 +989,12 @@ static int tile_handler_dirty(request_rec *r)
 	sconf = r->server->module_config;
 	scfg = ap_get_module_config(sconf, &tile_module);
 
+	// Is /dirty URL enabled?
+	if (!scfg->enableDirtyUrl) {
+		ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, r, "tile_handler_dirty: /dirty URL is not enabled");
+		return HTTP_NOT_FOUND;
+	}
+
 	if (scfg->bulkMode) {
 		return OK;
 	}
@@ -1130,6 +1140,8 @@ static int tile_storage_hook(request_rec *r)
 
 static int tile_handler_status(request_rec *r)
 {
+	ap_conf_vector_t *sconf;
+	tile_server_conf *scfg;
 	enum tileState state;
 	char mtime_str[APR_CTIME_LEN];
 	char atime_str[APR_CTIME_LEN];
@@ -1141,6 +1153,15 @@ static int tile_handler_status(request_rec *r)
 		return DECLINED;
 	}
 
+	sconf = r->server->module_config;
+	scfg = ap_get_module_config(sconf, &tile_module);
+
+	// Is /status URL enabled?
+	if (!scfg->enableStatusUrl) {
+		ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, r, "tile_handler_status: /status URL is not enabled");
+		return HTTP_NOT_FOUND;
+	}
+
 	rdata = (struct tile_request_data *)ap_get_module_config(r->request_config, &tile_module);
 	cmd = rdata->cmd;
 
@@ -2099,6 +2120,7 @@ static const char *load_tile_config(cmd_parms *cmd, void *mconfig, const char *c
 	const char * result;
 	char fileExtension[INILINE_MAX];
 	char mimeType[INILINE_MAX];
+	char outputFormat[INILINE_MAX];
 	char * description = NULL;
 	char * attribution = NULL;
 	char * cors = NULL;
@@ -2210,6 +2232,7 @@ static const char *load_tile_config(cmd_parms *cmd, void *mconfig, const char *c
 			strcpy(url, "");
 			strcpy(fileExtension, "png");
 			strcpy(mimeType, "image/png");
+			strcpy(outputFormat, "png256");
 			description = NULL;
 			cors = NULL;
 			attribution = NULL;
@@ -2289,7 +2312,7 @@ static const char *load_tile_config(cmd_parms *cmd, void *mconfig, const char *c
 					return "TYPE too long";
 				}
 
-				if (sscanf(value, "%[^ ] %[^;#]", fileExtension, mimeType) != 2) {
+				if (sscanf(value, "%[^ ] %[^ ] %[^;#]", fileExtension, mimeType, outputFormat) < 2) {
 					if (description) {
 						free(description);
 						description = NULL;
@@ -2666,6 +2689,20 @@ static const char *mod_tile_bulk_mode(cmd_parms *cmd, void *mconfig, int bulkMod
 	return NULL;
 }
 
+static const char *mod_tile_enable_status_url(cmd_parms *cmd, void *mconfig, int enableStatusUrl)
+{
+	tile_server_conf *scfg = ap_get_module_config(cmd->server->module_config, &tile_module);
+	scfg->enableStatusUrl = enableStatusUrl;
+	return NULL;
+}
+
+static const char *mod_tile_enable_dirty_url(cmd_parms *cmd, void *mconfig, int enableDirtyUrl)
+{
+	tile_server_conf *scfg = ap_get_module_config(cmd->server->module_config, &tile_module);
+	scfg->enableDirtyUrl = enableDirtyUrl;
+	return NULL;
+}
+
 static const char *mod_tile_delaypool_tiles_config(cmd_parms *cmd, void *mconfig, const char *bucketsize_string, const char *topuprate_string)
 {
 	int bucketsize;
@@ -2722,10 +2759,10 @@ static void *create_tile_config(apr_pool_t *p, server_rec *s)
 	scfg->max_load_old = MAX_LOAD_OLD;
 	scfg->max_load_missing = MAX_LOAD_MISSING;
 	scfg->veryold_threshold = VERYOLD_THRESHOLD;
-	strncpy(scfg->renderd_socket_name, RENDER_SOCKET, PATH_MAX - 1);
+	strncpy(scfg->renderd_socket_name, RENDERD_SOCKET, PATH_MAX - 1);
 	scfg->renderd_socket_name[PATH_MAX - 1] = 0;
 	scfg->renderd_socket_port = 0;
-	strncpy(scfg->tile_dir, HASH_PATH, PATH_MAX - 1);
+	strncpy(scfg->tile_dir, RENDERD_TILE_DIR, PATH_MAX - 1);
 	scfg->tile_dir[PATH_MAX - 1] = 0;
 	memset(&(scfg->cache_extended_hostname), 0, PATH_MAX);
 	scfg->cache_extended_duration = 0;
@@ -2745,6 +2782,8 @@ static void *create_tile_config(apr_pool_t *p, server_rec *s)
 	scfg->delaypoolRenderSize = AVAILABLE_RENDER_BUCKET_SIZE;
 	scfg->delaypoolRenderRate = RENDER_TOPUP_RATE;
 	scfg->bulkMode = 0;
+	scfg->enableStatusUrl = 1;
+	scfg->enableDirtyUrl = 1;
 
 
 	return scfg;
@@ -2787,6 +2826,8 @@ static void *merge_tile_config(apr_pool_t *p, void *basev, void *overridesv)
 	scfg->delaypoolRenderSize = scfg_over->delaypoolRenderSize;
 	scfg->delaypoolRenderRate = scfg_over->delaypoolRenderRate;
 	scfg->bulkMode = scfg_over->bulkMode;
+	scfg->enableStatusUrl = scfg_over->enableStatusUrl;
+	scfg->enableDirtyUrl = scfg_over->enableDirtyUrl;
 
 	//Construct a table of minimum cache times per zoom level
 	for (i = 0; i <= MAX_ZOOM_SERVER; i++) {
@@ -2978,6 +3019,20 @@ static const command_rec tile_cmds[] = {
 		OR_OPTIONS,                      /* where available */
 		"On Off - make all requests to renderd with bulk render priority, never mark tiles dirty"  /* directive description */
 	),
+	AP_INIT_FLAG(
+		"ModTileEnableStatusURL",        /* directive name */
+		mod_tile_enable_status_url,      /* config action routine */
+		NULL,                            /* argument to include in call */
+		OR_OPTIONS,                      /* where available */
+		"On Off - whether to handle .../status urls "  /* directive description */
+	),
+	AP_INIT_FLAG(
+		"ModTileEnableDirtyURL",         /* directive name */
+		mod_tile_enable_dirty_url,       /* config action routine */
+		NULL,                            /* argument to include in call */
+		OR_OPTIONS,                      /* where available */
+		"On Off - whether to handle .../dirty urls "  /* directive description */
+	),
 	{NULL}
 };
 
